@@ -1,7 +1,7 @@
-
 # -*- test-case-name: twisted.internet.test.test_endpoints -*-
 # Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
+
 """
 Implementations of L{IStreamServerEndpoint} and L{IStreamClientEndpoint} that
 wrap the L{IReactorTCP}, L{IReactorSSL}, and L{IReactorUNIX} interfaces.
@@ -12,22 +12,30 @@ parsed by the L{clientFromString} and L{serverFromString} functions.
 @since: 10.1
 """
 
+from __future__ import division, absolute_import
+
 import os, socket
 
-from zope.interface import implements, directlyProvides
+from zope.interface import implementer, directlyProvides
 import warnings
 
+from twisted.python.compat import _PY3
 from twisted.internet import interfaces, defer, error, fdesc, threads
 from twisted.internet.protocol import ClientFactory, Protocol
-from twisted.plugin import IPlugin, getPlugins
 from twisted.internet.interfaces import IStreamServerEndpointStringParser
 from twisted.internet.interfaces import IStreamClientEndpointStringParser
 from twisted.python.filepath import FilePath
 from twisted.python.systemd import ListenFDs
 from twisted.internet.abstract import isIPv6Address
-from twisted.internet import stdio
-from twisted.internet.stdio import PipeAddress
 
+if not _PY3:
+    from twisted.plugin import IPlugin, getPlugins
+    from twisted.internet import stdio
+    from twisted.internet.stdio import PipeAddress
+else:
+    from zope.interface import Interface
+    class IPlugin(Interface):
+        pass
 
 __all__ = ["clientFromString", "serverFromString",
            "TCP4ServerEndpoint", "TCP6ServerEndpoint",
@@ -36,6 +44,11 @@ __all__ = ["clientFromString", "serverFromString",
            "SSL4ServerEndpoint", "SSL4ClientEndpoint",
            "AdoptedStreamServerEndpoint", "StandardIOEndpoint"]
 
+__all3__ = ["TCP4ServerEndpoint", "TCP6ServerEndpoint",
+            "TCP4ClientEndpoint", "TCP6ClientEndpoint",
+            "SSL4ServerEndpoint", "SSL4ClientEndpoint",
+            ]
+
 
 class _WrappingProtocol(Protocol):
     """
@@ -213,11 +226,11 @@ class _WrappingFactory(ClientFactory):
 
 
 
+@implementer(interfaces.IStreamServerEndpoint)
 class StandardIOEndpoint(object):
     """
     A Standard Input/Output endpoint
     """
-    implements(interfaces.IStreamServerEndpoint)
 
     def __init__(self, reactor):
         """
@@ -235,11 +248,11 @@ class StandardIOEndpoint(object):
 
 
 
+@implementer(interfaces.IStreamServerEndpoint)
 class _TCPServerEndpoint(object):
     """
     A TCP server endpoint interface
     """
-    implements(interfaces.IStreamServerEndpoint)
 
     def __init__(self, reactor, port, backlog, interface):
         """
@@ -314,11 +327,11 @@ class TCP6ServerEndpoint(_TCPServerEndpoint):
 
 
 
+@implementer(interfaces.IStreamClientEndpoint)
 class TCP4ClientEndpoint(object):
     """
     TCP client endpoint with an IPv4 configuration.
     """
-    implements(interfaces.IStreamClientEndpoint)
 
     def __init__(self, reactor, host, port, timeout=30, bindAddress=None):
         """
@@ -360,6 +373,7 @@ class TCP4ClientEndpoint(object):
 
 
 
+@implementer(interfaces.IStreamClientEndpoint)
 class TCP6ClientEndpoint(object):
     """
     TCP client endpoint with an IPv6 configuration.
@@ -374,6 +388,7 @@ class TCP6ClientEndpoint(object):
     @ivar _GAI_ADDRESS_HOST: Index of the actual host-address in the
         5-tuple L{_GAI_ADDRESS}.
     """
+
     _getaddrinfo = socket.getaddrinfo
     _deferToThread = threads.deferToThread
     _GAI_ADDRESS = 4
@@ -410,7 +425,7 @@ class TCP6ClientEndpoint(object):
 
     def _nameResolution(self, host):
         """
-        Resolve the hostname string into a tuple containig the host
+        Resolve the hostname string into a tuple containing the host
         IPv6 address.
         """
         return self._deferToThread(
@@ -431,11 +446,11 @@ class TCP6ClientEndpoint(object):
 
 
 
+@implementer(interfaces.IStreamServerEndpoint)
 class SSL4ServerEndpoint(object):
     """
     SSL secured TCP server endpoint with an IPv4 configuration.
     """
-    implements(interfaces.IStreamServerEndpoint)
 
     def __init__(self, reactor, port, sslContextFactory,
                  backlog=50, interface=''):
@@ -446,7 +461,7 @@ class SSL4ServerEndpoint(object):
         @type port: int
 
         @param sslContextFactory: An instance of
-            L{twisted.internet._sslverify.OpenSSLCertificateOptions}.
+            L{twisted.internet.ssl.ContextFactory}.
 
         @param backlog: Size of the listen queue
         @type backlog: int
@@ -474,11 +489,11 @@ class SSL4ServerEndpoint(object):
 
 
 
+@implementer(interfaces.IStreamClientEndpoint)
 class SSL4ClientEndpoint(object):
     """
     SSL secured TCP client endpoint with an IPv4 configuration
     """
-    implements(interfaces.IStreamClientEndpoint)
 
     def __init__(self, reactor, host, port, sslContextFactory,
                  timeout=30, bindAddress=None):
@@ -492,7 +507,7 @@ class SSL4ClientEndpoint(object):
         @type port: int
 
         @param sslContextFactory: SSL Configuration information as an instance
-            of L{OpenSSLCertificateOptions}.
+            of L{twisted.internet.ssl.ContextFactory}.
 
         @param timeout: Number of seconds to wait before assuming the
             connection has failed.
@@ -526,13 +541,12 @@ class SSL4ClientEndpoint(object):
 
 
 
+@implementer(interfaces.IStreamServerEndpoint)
 class UNIXServerEndpoint(object):
     """
     UnixSocket server endpoint.
     """
-    implements(interfaces.IStreamServerEndpoint)
-
-    def __init__(self, reactor, address, backlog=50, mode=0666, wantPID=0):
+    def __init__(self, reactor, address, backlog=50, mode=0o666, wantPID=0):
         """
         @param reactor: An L{IReactorUNIX} provider.
         @param address: The path to the Unix socket file, used when listening
@@ -561,12 +575,11 @@ class UNIXServerEndpoint(object):
 
 
 
+@implementer(interfaces.IStreamClientEndpoint)
 class UNIXClientEndpoint(object):
     """
     UnixSocket client endpoint.
     """
-    implements(interfaces.IStreamClientEndpoint)
-
     def __init__(self, reactor, path, timeout=30, checkPID=0):
         """
         @param reactor: An L{IReactorUNIX} provider.
@@ -605,6 +618,7 @@ class UNIXClientEndpoint(object):
 
 
 
+@implementer(interfaces.IStreamServerEndpoint)
 class AdoptedStreamServerEndpoint(object):
     """
     An endpoint for listening on a file descriptor initialized outside of
@@ -757,14 +771,13 @@ def _parseSSL(factory, port, privateKey="server.pem", certKey=None,
 
 
 
+@implementer(IPlugin, IStreamServerEndpointStringParser)
 class _StandardIOParser(object):
     """
     Stream server endpoint string parser for the Standard I/O type.
 
     @ivar prefix: See L{IStreamClientEndpointStringParser.prefix}.
     """
-    implements(IPlugin, IStreamServerEndpointStringParser)
-
     prefix = "stdio"
 
     def _parseServer(self, reactor):
@@ -784,6 +797,7 @@ class _StandardIOParser(object):
 
 
 
+@implementer(IPlugin, IStreamServerEndpointStringParser)
 class _SystemdParser(object):
     """
     Stream server endpoint string parser for the I{systemd} endpoint type.
@@ -793,8 +807,6 @@ class _SystemdParser(object):
     @ivar _sddaemon: A L{ListenFDs} instance used to translate an index into an
         actual file descriptor.
     """
-    implements(IPlugin, IStreamServerEndpointStringParser)
-
     _sddaemon = ListenFDs.fromEnvironment()
 
     prefix = "systemd"
@@ -835,14 +847,13 @@ class _SystemdParser(object):
 
 
 
+@implementer(IPlugin, IStreamServerEndpointStringParser)
 class _TCP6ServerParser(object):
     """
     Stream server endpoint string parser for the TCP6ServerEndpoint type.
 
     @ivar prefix: See L{IStreamClientEndpointStringParser.prefix}.
     """
-    implements(IPlugin, IStreamServerEndpointStringParser)
-
     prefix = "tcp6"     # Used in _parseServer to identify the plugin with the endpoint type
 
     def _parseServer(self, reactor, port, backlog=50, interface='::'):
@@ -1000,7 +1011,7 @@ def _parseServer(description, factory, default=None):
     endpointType = args[0]
     parser = _serverParsers.get(endpointType)
     if parser is None:
-        # If the required parser is not found in _server, check if 
+        # If the required parser is not found in _server, check if
         # a plugin exists for the endpointType
         for plugin in getPlugins(IStreamServerEndpointStringParser):
             if plugin.prefix == endpointType:
@@ -1250,7 +1261,7 @@ def _parseClientUNIX(*args, **kwargs):
     Valid keyword arguments to this function are all L{IReactorUNIX.connectUNIX}
     keyword arguments except for C{checkPID}.  Instead, C{lockfile} is accepted
     and has the same meaning.  Also C{path} is used instead of C{address}.
-    
+
     Valid positional arguments to this function are C{path}.
 
     @return: The coerced values as a C{dict}.
@@ -1309,15 +1320,15 @@ def clientFromString(reactor, description):
 
         clientFromString(reactor, "ssl:host=web.example.com:port=443:"
                                   "caCertsDir=/etc/ssl/certs")
-    
+
     You can create a UNIX client endpoint with the 'path' argument and optional
     'lockfile' and 'timeout' arguments::
-    
+
         clientFromString(reactor, "unix:path=/var/foo/bar:lockfile=1:timeout=9")
-    
+
     or, with the path as a positional argument with or without optional
     arguments as in the following 2 examples::
-    
+
         clientFromString(reactor, "unix:/var/foo/bar")
         clientFromString(reactor, "unix:/var/foo/bar:lockfile=1:timeout=9")
 
@@ -1345,3 +1356,11 @@ def clientFromString(reactor, description):
         raise ValueError("Unknown endpoint type: %r" % (aname,))
     kwargs = _clientParsers[name](*args, **kwargs)
     return _endpointClientFactories[name](reactor, **kwargs)
+
+
+if _PY3:
+    for name in __all__[:]:
+        if name not in __all3__:
+            __all__.remove(name)
+            del globals()[name]
+    del name, __all3__
